PAGE 60,132
.8086
;------------------------ Parameters -----------------------------
TK_PrnBuffSize	EQU	4096
MaxPCXRunLength	EQU	03FH
PCX		EQU	0		; File Types
PCC		EQU	1		;  "     "
GX1		EQU	2		;  "     "
ReadToScreen	EQU	0		; Operations
WriteToFile	EQU	1		;  "     "
ReadToBuff	EQU	2		;  "     "
WriteFromBuff	EQU	3		;  "     "
;--- Error Codes
ErrNone		EQU	0
ErrOnOpen	EQU	1
ErrOnClose	EQU	2
ErrInvalidFile	EQU	3
ErrFileToBig	EQU	4
ErrReadingData	EQU	5
ErrWritingData	EQU	6
ErrInvalidOp	EQU	7
ErrInvalidData	EQU	8
;--- Register locations
GpxIR		EQU	03CEH		; Graphics controller	
SeqIR		EQU	03C4H		; Sequencer
ReadMapSelect	EQU	4		; GPX reg
MapMaskPtr	EQU	2		; SEQ reg
GpxModePtr	EQU	5		; GPX reg
;-------------------- Language Interface -------------------------
; Choices:
MICROSOFT	EQU	0
RYAN_MCFARLAND	EQU	1
TURBO_PASCAL	EQU	2
GPX_DRIVER	EQU	3
;
LANGUAGE	EQU	MICROSOFT
;-------------------- Include Macro Files ------------------------
if1
  include CSGMACRO.INC
endif
;-------------------- Code Segment Definition --------------------
START_CODE_SEG  LANGUAGE,TKFILE		; macro
;--- Global external variables
extrn EGAC16:far			; main toolkit entry point
extrn TK_PRINT_BUFF:BYTE		; 4096 bytes reserved
extrn TK_RSCREEN_BASE:WORD		; read enabled page
extrn TK_WSCREEN_BASE:WORD		; write enabled page
extrn TK_SCR_WID:WORD			; logical screen width
extrn TK_WRITE_MASK:BYTE		; write mask
extrn TK_READ_MASK:BYTE			; read mask
extrn TK_PHYS_HI:WORD			; physical page height
extrn TK_ICPX:WORD			; x current position
extrn TK_ICPY:WORD			; y current position
;---------------------- Constants -----------------------
OpenFile	EQU	03DH		; DOS function codes
CreateFile	EQU	03CH		;   "      "
CloseFile	EQU	03EH		;   "      "
ReadFile	EQU	03FH		;   "      "
WriteFile	EQU	040H		;   "      "
OpenRW		EQU	02H		; Read/Write Access
NoAttribute	EQU	0
;============== Data header for PCX and PCC data files =============
EVEN
PCX_HeadSize	EQU	128		; 128 byte header block
strt_head = $
PCX_Header	LABEL	BYTE
;----- constants
d_Manuf		db	?		; ?
d_Hard		db	?		; version info
		; 0 = 2.5
		; 2 = 2.8 w/palette info (or 3.0 w/o)
		; 3 = no palette info (2.8 or 3.0)
		; 5 = 3.0 with palette info
d_Encod		db	?		; run length encoding mode
;----- calculated
d_Bitpx		db	4		; bits per pixel
d_X1		dw	?		; picture dimensions
d_Y1		dw	?
d_X2		dw	?
d_Y2		dw	?
d_Hres		dw	?		; card res
d_Vres		dw	?
d_clrma		db	48 dup(0)	; palette
d_Vmode		db	0		; ignored
d_NPlanes	db	0		; number of planes
d_Bplin		dw	0		; bytes per line in picture
;----- filler
d_Xtra		db	(PCX_HeadSize-($-strt_head)) dup(?)
;===================== Working Variables ==========================
EVEN
SetupPtr	dd	?		; pointer to setup array
DataSeg		dw	?		; current data segment
FileStatus	dw	?
FileHandle	dw	?
FilePosition	dw	?
FileType	dw	?
UserBuffer	dd	?		; far pointer to user buffer
UserBufferIndex	dw	?		; pointer into user buffer
Operation	dw	?
IO_BuffSize	dw	?		; size of working I/O buffer
WorkBuffPtr	dw	?		; starting index in tk_print_buff
EndOfBuffPtr	dw	?
PageRowAddress	dw	?
BytesInRow	dw	?		; total bytes for all planes in row
BytesInPlane	dw	?		; bytes per row per plane
BytesToWrite	dw	?
TempByteCnt	dw	?
NumRows		dw	?
CurrRow		dw	?
ImageWidth	dw	?
ImageHeight	dw	?
PT		dw	10 dup (?)
IFUNC		dw	?,?
ReadMap		db	?
MapMask		db	?
;****************************************************************
; TKFILE *  Created: 10/8/87		Revised: 1/18/88	*
;*********  By: Edward W. Connell				*
;               Connell Scientific Graphics			*
;		PO Box 1765					*
;		Woodinville, WA 98072				*
;								*
; Copyright (C) Connell Scientific Graphics 1986, 1987		*
; All Rights Reserved.						*
;---------------------------------------------------------------*
; Syntax: tkfile( FileName, Setup )				*
;---------------------------------------------------------------*
; Entry:							*
;								*
; FileName  = ASCIIZ string					*
;								*
; Setup								*
;								*
;   (0)	FileType  = 0 .PCX,  1 .PCC,   2 .GX1			*
;								*
;   (1)	Operation = 0  Read file to screen		    	*
;		    1  Write screen to file			*
;		    2  Read file to data buffer	 (.PCC only)	*
;		    3  Write data buffer to file (.PCC only)	*
;								*
;   (2) Width     = width of region to read or write		*
;								*
;   (3) Height    = height of region to read or write		*
;								*
;   (4) Buff Offset  = offset of user buffer			*
;   (5) Buff Segment = segment of user buffer		      	*
;   								*
;****************************************************************
ENTERP	TKFILE,FarPro,<>
;----- Get and store current data segment and set file status
	mov	ax,ds
	mov	DataSeg,ax
	mov	FileStatus,ErrNone		; No error
;----- Save Pointer to setup array
	SET_ESOFBX 2,2
	mov	word ptr SetupPtr[0],bx
	mov	word ptr SetupPtr[2],es
;----- Save Pointer to user array
	mov	ax,es:[bx+8]			; offset
	mov	word ptr UserBuffer[0],ax
	mov	ax,es:[bx+10]			; segment
	mov	word ptr UserBuffer[2],ax
;----- Get file type
	mov	ax,es:[bx]
	mov	FileType,ax			
;----- Get Operation
	mov	ax,es:[bx+2]
	mov	Operation,ax
;----- Open File
	SET_ESOFBX 1,2
	test	Operation,01H
;	$IF	NZ
	JZ	tkfile$IF1
		mov	ah,CreateFile
		mov	cx,NoAttribute
;	$ELSE
	JMP SHORT tkfile$IF2
tkfile$IF1:
		mov	ah,OpenFile
		mov	al,OpenRW
;	$ENDIF
tkfile$IF2:
	mov	dx,bx				; offset of name
	push	ds
	push	es
	pop	ds
	int	21H
	pop	ds
;	$IF	C
	JNC	tkfile$IF3
		mov	FileStatus,ErrOnOpen
		jmp	TKFILE_exit
;	$ENDIF
tkfile$IF3:
	mov	FileHandle,ax
;===== Read/Write File Header
	cmp	FileType,GX1
;	$IF	E
	JNE	tkfile$IF4

		jmp	TKFILE_CompVars
;	$ENDIF
tkfile$IF4:
	test Operation,01H      ; PCX or PCC
;	$IF	NZ		; write operation - Build header
	JZ	tkfile$IF5
	   cmp	FileType,PCX
;	   $IF	E
	   JNE	tkfile$IF6
		   call BuildPCXHeader
		   ;---- Write header to file
	   	   mov	ah,WriteFile
   		   mov	bx,FileHandle
	   	   lea	dx,PCX_Header
	   	   mov	cx,PCX_HeadSize
	   	   int	21H
;	   	   $IF	C
	   	   JNC	tkfile$IF7
	   		mov	FileStatus,ErrReadingData
			jmp	TKFILE_exit
;   		   $ENDIF
tkfile$IF7:
;	   $ELSE
	   JMP SHORT tkfile$IF8
tkfile$IF6:
		   call BuildPCCHeader
		   ;---- Write header to file
	   	   mov	ah,WriteFile
   		   mov	bx,FileHandle
	   	   lea	dx,PCX_Header
	   	   mov	cx,PCX_HeadSize
	   	   int	21H
;	   	   $IF	C
	   	   JNC	tkfile$IF9
	   		mov	FileStatus,ErrReadingData
			jmp	TKFILE_exit
;   		   $ENDIF
tkfile$IF9:
;	   $ENDIF
tkfile$IF8:
;	$ELSE			; read operation - Load header
	JMP SHORT tkfile$IF10
tkfile$IF5:
	   mov	ah,ReadFile
	   mov	bx,FileHandle
	   lea	dx,PCX_Header
	   mov	cx,PCX_HeadSize
	   int	21H
;	   $IF	C
	   JNC	tkfile$IF11
	   	mov	FileStatus,ErrReadingData
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF11:
;	$ENDIF
tkfile$IF10:
;----- Compute some working variables
TKFILE_CompVars:
	;----- sort P1 and P2
	mov	ax,d_X2
	cmp	ax,d_X1
;	$IF	L
	JGE	tkfile$IF12
		xchg	ax,d_X1
		xchg	ax,d_X2
;	$ENDIF
tkfile$IF12:
	mov	ax,d_Y2
	cmp	ax,d_Y1
;	$IF	L
	JGE	tkfile$IF13
		xchg	ax,d_Y1
		xchg	ax,d_Y2
;	$ENDIF
tkfile$IF13:
	;-----
	les	bx,SetupPtr
	mov	ax,d_X2
	sub	ax,d_X1
	inc	ax
	mov	ImageWidth,ax
	mov	es:[bx+4],ax			; return to caller
	mov	ax,d_Y2
	sub	ax,d_Y1
	inc	ax
	mov	ImageHeight,ax
	mov	es:[bx+6],ax			; return to caller
;----- Branch to case
	cmp	FileType,PCX
;	$IF	E
	JNE	tkfile$IF14
	   cmp	Operation,ReadToScreen
;	   $IF	E
	   JNE	tkfile$IF15
	   	call	DecompressPCX
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF15:
	   cmp	Operation,WriteToFile
;	   $IF	E
	   JNE	tkfile$IF16
	   	call	CompressPCX
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF16:
	   ;---- Operation Error 
		mov	FileStatus,ErrInvalidOp
		jmp	TKFILE_exit
;	$ENDIF
tkfile$IF14:
	cmp	FileType,PCC
;	$IF	E
	JNE	tkfile$IF17
	   cmp	Operation,ReadToScreen
;	   $IF	E
	   JNE	tkfile$IF18
	   	call	DecompressPCC
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF18:
	   cmp	Operation,WriteToFile
;	   $IF	E
	   JNE	tkfile$IF19
	   	call	CompressPCC
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF19:
	   cmp	Operation,ReadToBuff
;	   $IF	E
	   JNE	tkfile$IF20
	   	call	DecompressPCC
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF20:
	   cmp	Operation,WriteFromBuff
;	   $IF	E
	   JNE	tkfile$IF21
	   	call	CompressPCC
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF21:
	   ;---- Operation Error 
		mov	FileStatus,ErrInvalidOp
		jmp	TKFILE_exit
;	$ENDIF
tkfile$IF17:
	cmp	FileType,GX1
;	$IF	E
	JNE	tkfile$IF22
	   cmp	Operation,ReadToScreen
;	   $IF	E
	   JNE	tkfile$IF23
;	   	call	DecompressGX1
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF23:
	   cmp	Operation,WriteToFile
;	   $IF	E
	   JNE	tkfile$IF24
;	   	call	CompressGX1
		jmp	TKFILE_exit
;	   $ENDIF
tkfile$IF24:
	   ;---- Operation Error 
		mov	FileStatus,ErrInvalidOp
		jmp	TKFILE_exit
;	$ENDIF
tkfile$IF22:
;----- 
TKFILE_exit:
	;---- Close Current File
	mov	ah,CloseFile
	mov	bx,FileHandle
	int	21H
;	$IF	C
	JNC	tkfile$IF25
		mov	FileStatus,ErrOnClose
;	$ENDIF
tkfile$IF25:
	;---- Reset write mask
	mov	dx,SeqIR
	mov	al,MapMaskPtr
	mov	ah,tk_write_mask
	out	dx,ax
	;----
        mov	ax,FileStatus
EXITP	TKFILE,FarPro,<>,2
;*****************************************************************
; CompressPCC  * Created: 10/13/87	Revised: 10/16/87	 *
;--------------* By: E. Connell			    		 *
;     This routine is used to compress the user specified region *
; or the user specified array to a .PCC image file.		 *
;----------------------------------------------------------------*
; Entry: Precomputed Values					 *
;       FileHandle    = Handle to current input file		 *
;	FileStatus    = Flag set after operation	      	 *
;	tk_scr_wid    = byte width of screen			 *
;	tk_print_buff = temporary I/O buffer			 *
;	d_Bplin       = Bytes per plane per row			 *
;---------------------------------------------------------------*
ENTERP	CompressPCC,NearPro,<>
  cld
  mov	dx,GpxIR
  mov	al,GpxModePtr
  mov	ah,0
  out	dx,ax					; R0W0
;------ Determine starting point in memory
  mov	ax,d_Bplin
  mov	BytesInPlane,ax
  mov	ax,tk_icpy
  mul	tk_scr_wid
  mov	PageRowAddress,ax
  mov	ax,tk_icpx
  shr	ax,1
  shr	ax,1
  shr	ax,1					; /8
  add	PageRowAddress,ax  
;------ Allocate working buffers
  lea	ax,tk_print_buff
  add	ax,TK_PrnBuffSize
  mov	EndOFBuffPtr,ax				; point past last byte
  mov	bx,BytesInPlane
  shl	bx,1
  shl	bx,1					; *4 planes
  sub	ax,bx
  mov	WorkBuffPtr,ax
;------ Main loop
  mov	CurrRow,0
  mov	cx,ImageHeight
;  $DO
tkfile$DO1:
  push	cx
;------ Get row and put into working buffer
	mov	ReadMap,0
	mov	MapMask,1
	mov	di,WorkBuffPtr
	mov	si,PageRowAddress
	mov	cx,4
;	$DO
tkfile$DO2:
	push	cx
	push	si
	;------ Select memory plane
	mov	dx,GpxIR
	mov	al,ReadMapSelect
	mov	ah,ReadMap
	inc	ReadMap
	out	dx,ax
	mov	ah,MapMask
	shl	MapMask,1
	test	tk_read_mask,ah
;	$IF	NZ
	JZ	tkfile$IF26
	  push	ds
	  mov	ax,tk_rscreen_base
	  mov	es,DataSeg
	  ;------ Move the data
	  mov	cx,BytesInPlane
	  shr	cx,1 				; word move is faster
	  mov	ds,ax
	  rep	movsw
	  pop	ds
;	$ELSE
	JMP SHORT tkfile$IF27
tkfile$IF26:
	   ;------ Fill with 0
	   mov	cx,tk_scr_wid
	   shr	cx,1 				; word move is faster
	   xor	ax,ax				; 0
	   rep	stos word ptr [di]
;	$ENDIF
tkfile$IF27:
	;-------
	pop	si
	pop	cx
;	$ENDDO	LOOP
	LOOP	tkfile$DO2
;------ Compress row and put into tk_print_buff
	mov	BytesToWrite,1
	mov	si,WorkBuffPtr
	lea	di,tk_print_buff
	mov	ah,[si]				; preload
	inc	si
	;
;	$DO
tkfile$DO3:
	lodsb					; get byte
	cmp	ah,al
;	$IF	E
	JNE	tkfile$IF28
		cmp	BytesToWrite,MaxPCXRunLength
;		$IF	E
		JNE	tkfile$IF29
			mov	byte ptr [di],0FFH	; count
			mov	[di+1],ah	 	; data
			add	di,2
			mov	BytesToWrite,1		; reset count
;		$ELSE
		JMP SHORT tkfile$IF30
tkfile$IF29:
			inc	BytesToWrite		; accumulate run
;		$ENDIF
tkfile$IF30:
		;------ Check if done
		cmp	si,EndOfBuffPtr
;		$IF	E
		JNE	tkfile$IF31
		   mov	al,byte ptr BytesToWrite
		   or	al,0C0H
		   mov	[di],al				; count
		   mov	[di+1],ah			; data
		   add	di,2
;		   $LEAVE
		   JMP	tkfile$LEAVE3
;		$ENDIF
tkfile$IF31:
;	$ELSE	;---- AH is run search data, AL is new byte
	JMP SHORT tkfile$IF32
tkfile$IF28:
		cmp	BytesToWrite,1
;		$IF	E
		JNE	tkfile$IF33
		    mov	 bl,ah
		    and  bl,0C0H
		    cmp  bl,0C0H
;		    $IF  E
		    JNE	tkfile$IF34
			mov  byte ptr [di],0C1H
			mov  [di+1],ah
			add  di,2
;		    $ELSE
		    JMP SHORT tkfile$IF35
tkfile$IF34:
		    	mov  [di],ah
			inc   di
;		    $ENDIF
tkfile$IF35:
;		$ELSE
		JMP SHORT tkfile$IF36
tkfile$IF33:
		    mov   bl,byte ptr BytesToWrite	; run length
		    or    bl,0C0H			; tag
		    mov   [di],bl			; count  			
		    mov   [di+1],ah			; data
		    add   di,2
		    mov   BytesToWrite,1	 	; reset
;		$ENDIF
tkfile$IF36:
		mov	ah,al			    	; Save search data
		;------ Check if done
		cmp	si,EndOfBuffPtr
;		$IF	E
		JNE	tkfile$IF37
		    mov	 bl,ah
		    and  bl,0C0H
		    cmp  bl,0C0H
;		    $IF  E
		    JNE	tkfile$IF38
			mov  byte ptr [di],0C1H
			mov  [di+1],ah
			add  di,2
;		    $ELSE
		    JMP SHORT tkfile$IF39
tkfile$IF38:
		    	mov  [di],ah
			inc   di			; advance counter
;		    $ENDIF
tkfile$IF39:
;		    $LEAVE
		    JMP	tkfile$LEAVE3
;		$ENDIF
tkfile$IF37:
;	$ENDIF
tkfile$IF32:
;	$ENDDO
	JMP	tkfile$DO3
tkfile$LEAVE3:
;------ Write compressed row to file
	mov	ah,WriteFile
	mov	bx,FileHandle
	lea	dx,tk_print_buff
	mov	cx,di				; end of buffer
	sub	cx,dx				; CX is size of buffer
	int	21H
;------ Next row
  mov	ax,tk_scr_wid
  add	PageRowAddress,ax			; Next screen row
  inc	CurrRow
  pop	cx
  dec	cx
;  $LEAVE CXZ
  JCXZ	tkfile$LEAVE1
;  $ENDDO
  JMP	tkfile$DO1
tkfile$LEAVE1:
EXITP	CompressPCC,NearPro,<>
;*****************************************************************
; DecompressPCC  * Created: 10/15/87	Revised: 10/29/87	 *
;----------------* By: E. Connell			     	 *
;     This routine is used to read a .PCC image file and write   *
; it to the current position.  Or to the user specified array.	 *
;----------------------------------------------------------------*
; Entry: Precomputed Values					 *
;       FileHandle    = Handle to current input file		 *
;	FileStatus    = Flag set after operation	      	 *
;	tk_scr_wid    = byte width of screen			 *
;	tk_print_buff = temporary I/O buffer			 *
;~---------------------------------------------------------------*
ENTERP	DecompressPCC,NearPro,<>
  ;---- Compute some working values
  mov	ax,d_Bplin
  mov	BytesInPlane,ax				; bytes per plane per row
  shl	ax,1
  shl	ax,1					; *4
  mov	BytesInRow,ax
  mov	ax,ImageHeight
  mov	NumRows,ax
  ;---- Determine starting point on screen
  mov	ax,tk_icpy
  mul	tk_scr_wid
  mov	PageRowAddress,ax
  mov	ax,tk_icpx
  shr	ax,1
  shr	ax,1
  shr	ax,1					; /8
  add	PageRowAddress,ax
  ;---- Allocate working buffers
  mov	ax,TK_PrnBuffSize
  sub	ax,BytesInRow
  mov	WorkBuffPtr,ax
  dec	ax
  mov	IO_BuffSize,ax
  lea	ax,tk_print_buff
  add	WorkBuffPtr,ax
  mov	UserBufferIndex,0
  ;---- Determine number of bytes to write
  ; Note: this is a guess.  The documentation on this format sucks.
  mov	cx,3
  mov	ax,d_X2
  sub	ax,d_X1
  inc	ax
  shr	ax,cl				; /8
  inc	ax				; +1 for phase alignment
  mov	BytesToWrite,ax
  ;---- Clip on X
  mov	bx,tk_icpx			
  shr	bx,cl				; /8
  add	ax,bx				; position + block width
  cmp	tk_scr_wid,ax
;  $IF	L
  JGE	tkfile$IF40
  	mov	ax,tk_scr_wid
	sub	ax,bx
;	$IF	S			; out of bounds
	JNS	tkfile$IF41
		mov	FileStatus,ErrInvalidOp
		jmp	DecompressPCC_exit
;	$ENDIF
tkfile$IF41:
	mov	BytesToWrite,ax
;  $ENDIF
tkfile$IF40:
  ;---- Clip On Y
  mov	bx,tk_icpy
  mov	ax,ImageHeight
  add	ax,bx				; position + block height
  cmp	tk_phys_hi,ax
;  $IF	L
  JGE	tkfile$IF42
  	mov	ax,tk_phys_hi
	sub	ax,bx
;	$IF	S
	JNS	tkfile$IF43
		mov	FileStatus,ErrInvalidOp
		jmp	DecompressPCC_exit
;	$ENDIF
tkfile$IF43:
	mov	NumRows,ax
;  $ENDIF
tkfile$IF42:
  ;---- Position on stack is saved so that routine may exit from
  ;     any loop level if an error condition occurs.
  mov	bp,sp				; save stack pointer
  cld
  mov	si,IO_BuffSize			; point past end to force data load
  mov	cx,NumRows
;  $DO
tkfile$DO4:
  push	cx
  ;------ Decode Row
  ;	  Note: DS:SI always points to next available file
  ;	        byte that is sitting in temporary buffer.	
  ;
  push	ds
  pop	es
  mov	di,WorkBuffPtr
  mov	ax,BytesInRow
  mov	TempByteCnt,ax
;  $DO
tkfile$DO5:
	;------ Check and load data buffer
	cmp	si,IO_BuffSize
;	$IF	GE
	JL	tkfile$IF44
	  mov   ah,ReadFile
	  mov   bx,FileHandle
	  mov   dx,offset tk_print_buff
	  mov   cx,IO_BuffSize
	  int   21H					; fill buffer
	  cmp   ax,0
;	  $IF   E					; file error, Attempt
	  JNE	tkfile$IF45
		mov	FileStatus,ErrReadingData	; to read more data
		jmp	DecompressPCC_exit		; than is available
;	  $ENDIF
tkfile$IF45:
	  mov   si,0					; reset pointer
;	$ENDIF
tkfile$IF44:
	;------ Decode
	mov	cx,1					; default
	mov	al,tk_print_buff[si]		; get data byte
	and	al,0C0H				; check top two
	cmp	al,0C0H
;	$IF	Z
	JNZ	tkfile$IF46
		mov	cl,tk_print_buff[si]	; get byte
		and	cx,003FH		; count
		inc	si
		cmp	si,IO_BuffSize
;		$IF	GE
		JL	tkfile$IF47
		  push	cx
		  mov   ah,ReadFile
		  mov   bx,FileHandle
		  mov   dx,offset tk_print_buff
		  mov   cx,IO_BuffSize
		  int   21H				; fill buffer
		  cmp   ax,0
;		  $IF   E				; file error, Attempt
		  JNE	tkfile$IF48
		     mov  FileStatus,ErrReadingData  	; to read more data
		     jmp  DecompressPCC_exit		; than is available
;		  $ENDIF
tkfile$IF48:
		  mov   si,0				; reset pointer
		  pop	cx
;		$ENDIF
tkfile$IF47:
;	$ENDIF
tkfile$IF46:
	mov	al,tk_print_buff[si]			; data byte
	;==== CX is count, AL is data
	push	cx
	rep	stosb					; store data
	pop	cx
	inc	si					; next file byte
	sub	TempByteCnt,cx	
;	$LEAVE	Z
	JZ	tkfile$LEAVE5
;	$IF	S					; Invalid Row
	JNS	tkfile$IF49
		mov	FileStatus,ErrInvalidData
		jmp	DecompressPCC_exit
;	$ENDIF
tkfile$IF49:
;  $ENDDO
  JMP	tkfile$DO5
tkfile$LEAVE5:
;============== Move data to target ===============
  cmp	Operation,ReadToScreen
;  $IF	E
  JNE	tkfile$IF50
	push	si				; save file position
	mov	es,tk_wscreen_base
	mov	di,PageRowAddress
	mov	si,WorkBuffPtr
	mov	dx,SeqIR
	mov	al,MapMaskPtr
	mov	ah,1
	mov	cx,4
;	$DO
tkfile$DO6:
	push	cx
	push	di
	push	si
	test	ah,tk_write_mask
;	$IF	NZ
	JZ	tkfile$IF51
		;---- Select memory plane
		out	dx,ax
		;---- write data
		mov	cx,BytesToWrite
		rep	movsb
		;----
;	$ENDIF
tkfile$IF51:
	shl	ah,1			      	; next plane
	pop	si
	add	si,BytesInPlane
	pop	di
	pop	cx
;	$ENDDO  LOOP
	LOOP	tkfile$DO6
	add	di,tk_scr_wid
	mov	PageRowAddress,di	      	; setup for next row
	pop	si			      	; restore file position
;  $ELSE			    			
  JMP SHORT tkfile$IF52
tkfile$IF50:
  ;---- Translate row data to bit block format and
  ;	put into user array at the current position.
  ;
  	call	XlatPCC2BBLK
;  $ENDIF
tkfile$IF52:
;------
pop	cx
dec	cx
;$LEAVE	CXZ
JCXZ	tkfile$LEAVE4
;$ENDDO
JMP	tkfile$DO4
tkfile$LEAVE4:
DecompressPCC_exit:
	mov	sp,bp				; restore position on stack
EXITP	DecompressPCC,NearPro,<>
;*****************************************************************
; CompressPCX  * Created: 10/13/87	Revised: 10/15/87	 *
;--------------* By: E. Connell			    		 *
;     This routine is used to write the current read enabled	 *
; page to a .PCX data file					 *
;----------------------------------------------------------------*
; Entry: Precomputed Values					 *
;       FileHandle    = Handle to current input file		 *
;	FileStatus    = Flag set after operation	      	 *
;	tk_scr_wid    = byte width of screen			 *
;	tk_print_buff = temporary I/O buffer			 *
;---------------------------------------------------------------*
ENTERP	CompressPCX,NearPro,<>
  cld
  mov	dx,GpxIR
  mov	al,GpxModePtr
  mov	ah,0
  out	dx,ax					; R0W0
;------ Allocate working buffers
  mov	PageRowAddress,0
  lea	ax,tk_print_buff
  add	ax,TK_PrnBuffSize
  mov	EndOFBuffPtr,ax				; point past last byte
  mov	bx,tk_scr_wid
  shl	bx,1
  shl	bx,1					; *4 planes
  sub	ax,bx
  mov	WorkBuffPtr,ax
;------ Main loop
  mov	CurrRow,0
  mov	cx,ImageHeight
;  $DO
tkfile$DO7:
  push	cx
;------ Get row and put into working buffer
	mov	ReadMap,0
	mov	MapMask,1
	mov	di,WorkBuffPtr
	mov	si,PageRowAddress
	mov	cx,4
;	$DO
tkfile$DO8:
	push	cx
	push	si
	;------ Select memory plane
	mov	dx,GpxIR
	mov	al,ReadMapSelect
	mov	ah,ReadMap
	inc	ReadMap
	out	dx,ax
	mov	ah,MapMask
	shl	MapMask,1
	test	tk_read_mask,ah
;	$IF	NZ
	JZ	tkfile$IF53
	  push	ds
	  mov	ax,tk_rscreen_base
	  mov	es,DataSeg
	  ;------ Move the data
	  mov	cx,tk_scr_wid
	  shr	cx,1 				; word move is faster
	  mov	ds,ax
	  rep	movsw
	  pop	ds
;	$ELSE
	JMP SHORT tkfile$IF54
tkfile$IF53:
	   ;------ Fill with 0
	   mov	cx,tk_scr_wid
	   shr	cx,1 				; word move is faster
	   xor	ax,ax				; 0
	   rep	stos word ptr [di]
;	$ENDIF
tkfile$IF54:
	;-------
	pop	si
	pop	cx
;	$ENDDO	LOOP
	LOOP	tkfile$DO8
;------ Compress row and put into tk_print_buff
	mov	BytesToWrite,1
	mov	si,WorkBuffPtr
	lea	di,tk_print_buff
	mov	ah,[si]				; preload
	inc	si
	;
;	$DO
tkfile$DO9:
	lodsb					; get byte
	cmp	ah,al
;	$IF	E
	JNE	tkfile$IF55
		cmp	BytesToWrite,MaxPCXRunLength
;		$IF	E
		JNE	tkfile$IF56
			mov	byte ptr [di],0FFH	; count
			mov	[di+1],ah	 	; data
			add	di,2
			mov	BytesToWrite,1		; reset count
;		$ELSE
		JMP SHORT tkfile$IF57
tkfile$IF56:
			inc	BytesToWrite		; accumulate run
;		$ENDIF
tkfile$IF57:
		;------ Check if done
		cmp	si,EndOfBuffPtr
;		$IF	E
		JNE	tkfile$IF58
		   mov	al,byte ptr BytesToWrite
		   or	al,0C0H
		   mov	[di],al				; count
		   mov	[di+1],ah			; data
		   add	di,2
;		   $LEAVE
		   JMP	tkfile$LEAVE9
;		$ENDIF
tkfile$IF58:
;	$ELSE	;---- AH is run search data, AL is new byte
	JMP SHORT tkfile$IF59
tkfile$IF55:
		cmp	BytesToWrite,1
;		$IF	E
		JNE	tkfile$IF60
		    mov	 bl,ah
		    and  bl,0C0H
		    cmp  bl,0C0H
;		    $IF  E
		    JNE	tkfile$IF61
			mov  byte ptr [di],0C1H
			mov  [di+1],ah
			add  di,2
;		    $ELSE
		    JMP SHORT tkfile$IF62
tkfile$IF61:
		    	mov  [di],ah
			inc   di
;		    $ENDIF
tkfile$IF62:
;		$ELSE
		JMP SHORT tkfile$IF63
tkfile$IF60:
		    mov   bl,byte ptr BytesToWrite	; run length
		    or    bl,0C0H			; tag
		    mov   [di],bl			; count  			
		    mov   [di+1],ah			; data
		    add   di,2
		    mov   BytesToWrite,1	 	; reset
;		$ENDIF
tkfile$IF63:
		mov	ah,al			    	; Save search data
		;------ Check if done
		cmp	si,EndOfBuffPtr
;		$IF	E
		JNE	tkfile$IF64
		    mov	 bl,ah
		    and  bl,0C0H
		    cmp  bl,0C0H
;		    $IF  E
		    JNE	tkfile$IF65
			mov  byte ptr [di],0C1H
			mov  [di+1],ah
			add  di,2
;		    $ELSE
		    JMP SHORT tkfile$IF66
tkfile$IF65:
		    	mov  [di],ah
			inc   di			; advance counter
;		    $ENDIF
tkfile$IF66:
;		    $LEAVE
		    JMP	tkfile$LEAVE9
;		$ENDIF
tkfile$IF64:
;	$ENDIF
tkfile$IF59:
;	$ENDDO
	JMP	tkfile$DO9
tkfile$LEAVE9:
;------ Write compressed row to file
	mov	ah,WriteFile
	mov	bx,FileHandle
	lea	dx,tk_print_buff
	mov	cx,di				; end of buffer
	sub	cx,dx				; CX is size of buffer
	int	21H
;------ Next row
  mov	ax,tk_scr_wid
  add	PageRowAddress,ax			; Next screen row
  inc	CurrRow
  pop	cx
  dec	cx
;  $LEAVE CXZ
  JCXZ	tkfile$LEAVE7
;  $ENDDO
  JMP	tkfile$DO7
tkfile$LEAVE7:
EXITP	CompressPCX,NearPro,<>
;*****************************************************************
; DecompressPCX * Created: 8/7/87	Revised: 10/09/87	 *
;----------------* By: E. Connell			    	 *
;     This routine is used to read a .PCX data file and write	 *
; it to the current write enabled page.				 *
;----------------------------------------------------------------*
; Entry: Precomputed Values					 *
;       FileHandle    = Handle to current input file		 *
;	FileStatus    = Flag set after operation	      	 *
;	tk_scr_wid    = byte width of screen			 *
;	tk_print_buff = temporary I/O buffer			 *
;---------------------------------------------------------------*
ENTERP	DecompressPCX,NearPro,<>
  ;---- Compute some working values
  mov	ax,ImageWidth				; 8 pixels per byte
  shr	ax,1					; /8 pix * 4 planes
  mov	BytesInRow,ax
  mov	ax,ImageHeight
  mov	NumRows,ax
  ;---- Allocate working buffers
  mov	ax,TK_PrnBuffSize
  sub	ax,BytesInRow
  mov	WorkBuffPtr,ax
  dec	ax
  mov	IO_BuffSize,ax
  lea	ax,tk_print_buff
  add	WorkBuffPtr,ax
  mov	ax,BytesInRow
  shr	ax,1
  shr	ax,1	       			; /4
  mov	BytesInPlane,ax			; bytes per plane per row
  ;---- Determine starting point on screen
  mov	ax,tk_icpy
  mul	tk_scr_wid
  mov	PageRowAddress,ax
  mov	ax,tk_icpx
  shr	ax,1
  shr	ax,1
  shr	ax,1					; /8
  add	PageRowAddress,ax
  ;---- Determine Number of bytes to write
  mov	ax,tk_scr_wid
  cmp	ax,BytesInPlane
;  $IF	L
  JGE	tkfile$IF67
  	mov	BytesToWrite,ax
;  $ELSE
  JMP SHORT tkfile$IF68
tkfile$IF67:
  	mov	ax,BytesInPlane
	mov	BytesToWrite,ax
;  $ENDIF
tkfile$IF68:
  ;---- Clip on X
  mov	cx,3
  mov	bx,tk_icpx			
  shr	bx,cl				; /8
  add	ax,bx				; position + block width
  cmp	tk_scr_wid,ax
;  $IF	L
  JGE	tkfile$IF69
  	mov	ax,tk_scr_wid
	sub	ax,bx
;	$IF	S			; out of bounds
	JNS	tkfile$IF70
		mov	FileStatus,ErrInvalidOp
		jmp	DecompressPCC_exit
;	$ENDIF
tkfile$IF70:
	mov	BytesToWrite,ax
;  $ENDIF
tkfile$IF69:
  ;---- Clip On Y
  mov	bx,tk_icpy
  mov	ax,ImageHeight
  add	ax,bx				; position + block height
  cmp	tk_phys_hi,ax
;  $IF	L
  JGE	tkfile$IF71
  	mov	ax,tk_phys_hi
	sub	ax,bx
;	$IF	S
	JNS	tkfile$IF72
		mov	FileStatus,ErrInvalidOp
		jmp	DecompressPCC_exit
;	$ENDIF
tkfile$IF72:
	mov	NumRows,ax
;  $ENDIF
tkfile$IF71:
  ;---- Position on stack is saved so that routine may exit from
  ;     any loop level if an error condition occurs.
  mov	bp,sp				; save stack pointer
  cld
  mov	si,IO_BuffSize			; point past end to force data load
  mov	cx,NumRows
;  $DO
tkfile$DO10:
  push	cx
  ;------ Decode Row
  ;	  Note: DS:SI always points to next available file
  ;	        byte that is sitting in temporary buffer.	
  ;
  push	ds
  pop	es
  mov	di,WorkBuffPtr
  mov	ax,BytesInRow
  mov	TempByteCnt,ax
;  $DO
tkfile$DO11:
	;------ Check and load data buffer
	cmp	si,IO_BuffSize
;	$IF	GE
	JL	tkfile$IF73
	  mov   ah,ReadFile
	  mov   bx,FileHandle
	  mov   dx,offset tk_print_buff
	  mov   cx,IO_BuffSize
	  int   21H					; fill buffer
	  cmp   ax,0
;	  $IF   E					; file error, Attempt
	  JNE	tkfile$IF74
		mov	FileStatus,ErrReadingData	; to read more data
		jmp	DecompressPCX_exit		; than is available
;	  $ENDIF
tkfile$IF74:
	  mov   si,0					; reset pointer
;	$ENDIF
tkfile$IF73:
	;------ Decode
	mov	cx,1					; default
	mov	al,tk_print_buff[si]		; get data byte
	and	al,0C0H				; check top two
	cmp	al,0C0H
;	$IF	Z
	JNZ	tkfile$IF75
		mov	cl,tk_print_buff[si]	; get byte
		and	cx,003FH		; count
		inc	si
		cmp	si,IO_BuffSize
;		$IF	GE
		JL	tkfile$IF76
		  push	cx
		  mov   ah,ReadFile
		  mov   bx,FileHandle
		  mov   dx,offset tk_print_buff
		  mov   cx,IO_BuffSize
		  int   21H				; fill buffer
		  cmp   ax,0
;		  $IF   E				; file error, Attempt
		  JNE	tkfile$IF77
		     mov  FileStatus,ErrReadingData  	; to read more data
		     jmp  DecompressPCX_exit		; than is available
;		  $ENDIF
tkfile$IF77:
		  mov   si,0				; reset pointer
		  pop	cx
;		$ENDIF
tkfile$IF76:
;	$ENDIF
tkfile$IF75:
	mov	al,tk_print_buff[si]			; data byte
	;==== CX is count, AL is data
	push	cx
	rep	stosb					; store data
	pop	cx
	inc	si					; next file byte
	sub	TempByteCnt,cx	
;	$LEAVE	Z
	JZ	tkfile$LEAVE11
;  $ENDDO
  JMP	tkfile$DO11
tkfile$LEAVE11:
  ;============== Move data to screen ===============
  push	si				; save file position
  mov	es,tk_wscreen_base
  mov	di,PageRowAddress
  mov	si,WorkBuffPtr
  mov	dx,SeqIR
  mov	al,MapMaskPtr
  mov	ah,1
  mov	cx,4
;  $DO
tkfile$DO12:
  push	cx
  push	di
  push	si
  test	ah,tk_write_mask
;  $IF	NZ
  JZ	tkfile$IF78
	;---- Select memory plane
	out	dx,ax
	;---- write data
	mov	cx,BytesToWrite
	rep	movsb
	;----
;  $ENDIF
tkfile$IF78:
  shl	ah,1					; next plane
  pop	si
  add	si,BytesInPlane
  pop	di
  pop	cx
;  $ENDDO  LOOP
  LOOP	tkfile$DO12
  add	di,tk_scr_wid
  mov	PageRowAddress,di			; setup for next row
  pop	si					; restore file position
;------
pop	cx
dec	cx
;$LEAVE	CXZ
JCXZ	tkfile$LEAVE10
;$ENDDO
JMP	tkfile$DO10
tkfile$LEAVE10:
DecompressPCX_exit:
	mov	sp,bp				; restore position on stack
EXITP	DecompressPCX,NearPro,<>
;****************************************************************
; Sub Procedure BuildPCXHeader *				*
;------------------------------*				*
;     This routine is used to create the PCX header for the     *
; output file.							*
;---------------------------------------------------------------*
ENTERP	BuildPCXHeader,NearPro,<>
;----- Flush header
	mov	cx,PCX_Headsize
	mov	al,0
	lea	di,PCX_Header
	mov	es,DataSeg
	rep	stosb
;----- Load Palette Vars
	mov	PT[0],0
	mov	IFUNC,3
	lea	di,d_clrma
	mov	bx,55H
	mov	cx,16
;	$DO
tkfile$DO13:
	lea	ax,IFUNC
	push	ds
	push	ax
	lea	ax,PT
	push	ds
	push	ax
	call	EGAC16
	;---- Red
	  mov	ax,PT[2]
	  mul	bx
	  mov	[di],al
	;---- Green
	  mov	ax,PT[4]
	  mul	bx
	  mov	[di+1],al
	;---- Blue
	  mov	ax,PT[6]
	  mul	bx
	  mov	[di+2],al
	;---- next
	add	di,3
	inc	PT[0]
;	$ENDDO	LOOP
	LOOP	tkfile$DO13
;----- Load dimensions
    	mov	ax,tk_scr_wid
 	mov	d_Bplin,ax
 	shl	ax,1
 	shl	ax,1
 	shl	ax,1			; *8
 	dec	ax
 	mov	d_Hres,004BH		; ?
 	mov	d_X1,0
 	mov	d_X2,ax
 	mov	ax,tk_phys_hi
 	dec	ax
 	mov	d_Vres,004BH		; ?
 	mov	d_Y1,0
 	mov	d_Y2,ax
;----- Load others
	mov	d_Manuf,0AH
	mov	d_Encod,01H
	mov	d_Hard,5 		; version
	mov	d_Bitpx,1		; bits per pixel
	mov	d_Vmode,0
	mov	d_NPlanes,4
EXITP	BuildPCXHeader,NearPro,<>
;****************************************************************
; Sub Procedure BuildPCCHeader *				*
;------------------------------*				*
;     This routine is used to create the PCC header for the     *
; output file.							*
;---------------------------------------------------------------*
ENTERP	BuildPCCHeader,NearPro,<>
;----- Flush header
	mov	cx,PCX_Headsize
	mov	al,0
	lea	di,PCX_Header
	mov	es,DataSeg
	rep	stosb
;----- Load Palette Vars
	mov	PT[0],0
	mov	IFUNC,3
	lea	di,d_clrma
	mov	bx,55H
	mov	cx,16
;	$DO
tkfile$DO14:
	lea	ax,IFUNC
	push	ds
	push	ax
	lea	ax,PT
	push	ds
	push	ax
	call	EGAC16
	;---- Red
	  mov	ax,PT[2]
	  mul	bx
	  mov	[di],al
	;---- Green
	  mov	ax,PT[4]
	  mul	bx
	  mov	[di+1],al
	;---- Blue
	  mov	ax,PT[6]
	  mul	bx
	  mov	[di+2],al
	;---- next
	add	di,3
	inc	PT[0]
;	$ENDDO	LOOP
	LOOP	tkfile$DO14
;----- Load dimensions
	les	bx,SetupPtr
        mov	ax,tk_icpx
	mov	d_X1,ax
	add	ax,es:[bx+4]			; region width
	dec	ax
	mov	d_X2,ax
	;
        mov	ax,tk_icpy
	mov	d_Y1,ax
	add	ax,es:[bx+6]			; region height
	dec	ax
	mov	d_Y2,ax
	;--- Determine Bytes per line
	mov	cl,4
	mov	si,d_X2
	mov	di,d_X1
	shr	si,cl
	shr	di,cl
	sub	si,di
	inc	si
	shl	si,1
	mov	d_Bplin,si
;----- Load others
	mov	d_Manuf,0AH
	mov	d_Encod,01H
	mov	d_Hres,04BH		; ?
	mov	d_Vres,04BH		; ?
	mov	d_Hard,5 		; version
	mov	d_Bitpx,1		; bits per pixel
	mov	d_Vmode,0
	mov	d_NPlanes,4
EXITP	BuildPCCHeader,NearPro,<>
;****************************************************************
; XlatPCC2BBLK * Created: 10/19/87	Revised:		*
;--------------* By: E. Connell					*
;     This routine is used to take a row of bit plane data and  *
; convert it to the toolkit bit block format.			*
;---------------------------------------------------------------*
; Entry:							*
;  UserBuffer      = far pointer to user specified data buffer  *
;  UserBufferIndex = offset into buffer to store data		*
;  WorkBuffPtr     = offset of source data within data segment	*
;  BytesInPlane    = number of bytes per plane row		*
;---------------------------------------------------------------*
; Exit:								*
;  UserBufferIndex = updated to next available storage		*
;		     position.					*
;~***************************************************************
ENTERP	XlatPCC2BBLK,NearPro,<ax,bx,cx,dx,si,di,es>
   mov	bx,1001H			; set mask
   mov	si,WorkBuffPtr			; source ds:[si]
   les	di,UserBuffer			
   add	di,UserBufferIndex		; dest   es:[di]
   ;------ Zero Out Buffer
   push	di
   mov	cx,BytesInRow
   shr	cx,1
   xor	ax,ax				; 0
   rep	stosw
   pop	di
   ;------
   mov	al,[si]				; get initial data byte
   mov	cx,4
;   $DO
tkfile$DO15:
   push	cx
   ;------
   	push	di
	mov	cx,BytesInPlane
;        $DO
tkfile$DO16:
	push	cx
	;---------
	  mov	cx,4
;	  $DO
tkfile$DO17:
	  ;---------
	      rcl	al,1
;	      $IF	C
	      JNC	tkfile$IF79
	          or	es:[di],bh
;	      $ENDIF
tkfile$IF79:
	      rcl	al,1
;	      $IF	C
	      JNC	tkfile$IF80
	      	  or	es:[di],bl
;	      $ENDIF
tkfile$IF80:
	      inc	di		; next dest
	  ;---------
;	  $ENDDO  LOOP
	  LOOP	tkfile$DO17
	  inc	si			; next source byte
	  mov	al,[si]
	;---------
	pop	cx
;	$ENDDO  LOOP
	LOOP	tkfile$DO16
	shl	bx,1			; next plane position
	pop	di			; next pass over destination
   ;------
   pop  cx
;   $ENDDO  LOOP
   LOOP	tkfile$DO15
   ;------- Advance pointer for next row
   mov	 ax,BytesInRow
   add   UserBufferIndex,ax
   ;
EXITP	XlatPCC2BBLK,NearPro,<es,di,si,dx,cx,bx,ax>
;****************************************************************
; XlatBBLK2PCC * Created: 10/19/87	Revised:		*
;--------------* By: E. Connell					*
;     This routine is used to take a row of bit block data and  *
; convert it to a row of seperated plane data.			*
;****************************************************************
ENTERP	XlatBBLK2PCC,NearPro,<>

EXITP	XlatBBLK2PCC,NearPro,<>
;****************************************************************
END_CODE_SEG
	end

